-- Target
--[[
Drathals HUD2 (c) 2007 by Horacio Hoyos / Arcanefoam
Drathals HUD (c) 2006 by Caeryn Dryad
Drathals HUD (c) 2006 by Markus Inger / Drathal / Silberklinge / Silbersegen

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are 
met:

    * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above 
       copyright notice, this list of conditions and the following 
       disclaimer in the documentation and/or other materials provided 
       with the distribution.
    * Neither the name of the DHUD2 Development Team nor the names of 
       its contributors may be used to endorse or promote products derived 
       from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]
local DHUD2, self = DHUD2, DHUD2
local _G = _G
local string_sub = string.sub
local string_gsub = string.gsub
local string_len = string.len
local string_format = string.format
local math_floor = math.floor
local styles = {
    values = {
        ["Full"] = "<value>/<maxval><percent>",
        ["Simple"] = "<value>/<maxval>",
        ["Small"] = "<value><percent>",
        ["Percent"] = "<percent>",
        ["Minimal"] = "<value>",
        },
    names = {
        ["Full"] = "<level><elite> <name> [ <class><pet><npc> ]",
        ["Simple"] = "<level><elite> <name>",
        ["Small"] = "<name> [ <class><pet><npc> ]",
        ["Minimal"] = "<level> <name>",
    },
}
local classColors = {
    ["HUNTER"] = "ABD473", 
    ["WARLOCK"] = "9482C9",
    ["PRIEST"] = "FFFFFF",
    ["PALADIN"] = "F58CBA",
    ["MAGE"] = "69CCF0",
    ["ROGUE"] = "FFF569",
    ["DRUID"] = "FF7D0A",
    ["SHAMAN"] = "00DBBA",
    ["WARRIOR"] = "C79C6E",
}
local defaultfont = "Fonts/FRIZQT__.TTF"
local defaultfont_num = "Interface\\AddOns\\DHUD\\layout\\Number.TTF"


local getReactionColor
do
    local reacColors = {
        "ff0000",
        "ffff00",
        "55ff55",
        "6d6dff",
        "008800",
        "cccccc",
        }
    function getReactionColor(unit)
    	local color
        if (UnitIsTapped(unit) and (not UnitIsTappedByPlayer(unit))) then
    		color = 6
    	elseif (UnitIsPlayer(unit)) then
    		if (UnitIsPVP(unit)) then
                color = UnitCanAttack("player", unit) and 1 or 5
    		else
    			if (UnitCanAttack("player", unit) or UnitFactionGroup(unit) ~= UnitFactionGroup("player")) then
    				color = 2
    			else
    				color = 4
    			end
    		end
    	else
    		local reaction = UnitReaction(unit, "player");
    		if (reaction) then
    			if (reaction < 4) then
    				color = 1
    			elseif (reaction == 4) then
    				color = 2
    			else
    				color = 3
    			end
    		end
    	end
    	return reacColors[color];
    end
    
end

function DHUD2:updateBarText(bar, maxval, curval, frac, color)
    local p = self.db.profile
    local percent = math_floor(frac * 100)
    local font = _G["DHUD2_"..bar.."_Text_Text"]
    local text = styles.values[p.styles[bar]]
    text = string_gsub(text, "<value>", "|cff"..color..curval)
    if text:find("|c") then
        text = string_gsub(text, "<maxval>", maxval)
    else
        text = string_gsub(text, "<maxval>", "|cff"..color..maxval)
    end
    if text:find("|c") then
        text = string_gsub(text, "<percent>", "|r |cff747070(|r"..percent.."|cff747070)|r%%")
    else
        text = string_gsub(text, "<percent>", "|cff747070(|r"..percent.."|cff747070)|r%%")
    end
    font:SetText(text)
    font:SetWidth(2000)
    font:SetJustifyH("RIGHT")
    local w = font:GetStringWidth() + 10
    font:SetWidth(w)
    _G["DHUD2_"..bar.."_Text"]:SetWidth(w)
    _G["DHUD2_"..bar.."_Text"]:Show()
end

function DHUD2:hideBarText(bar)
    _G["DHUD2_"..bar.."_Text"]:Hide()
end



--Upodate Target and TargetTarget text
function DHUD2:updateText(unit, show)
    if show then
        local p = self.db.profile
        local font = _G["DHUD2_"..unit.."_Text_Text"]
        local text = styles.names[p.styles[unit]]
        local level = UnitLevel(unit)
        local levelText, addLevel
        levelText = level < 0 and "??" or level
        level = level < 0 and 99 or level
        local levColor = GetDifficultyColor(level)
        local color = self:rgb2hex(levColor.r, levColor.g, levColor.b)
        text, addLevel = string_gsub(text, "<level>", "|cff"..color..levelText)
        local _, eliteText = self:checkElite(unit)
        if addLevel == 1 then
            text = string_gsub(text, "<elite>", ""..eliteText.."|r")
        else
            text = string_gsub(text, "<elite>", "|cff"..color..eliteText.."|r")
        end
        text = string_gsub(text, "<name>", "|cff"..getReactionColor(unit)..UnitName(unit).."|r")
        local localClass, class = UnitClass(unit)
        color = classColors[class]        
        if not(self:unitIsNPC(unit) or self:unitIsPet(unit)) then            
            if not UnitIsPlayer(unit) then
                localClass = UnitCreatureType(unit)
            end
            text = string_gsub(text, "<class>", "|cff"..color..localClass.."|r")
        else
            text = string_gsub(text, "<class>", "")
        end
        if self:unitIsPet(unit) then
            text = string_gsub(text, "<pet>", "|cff"..color.."Pet".."|r")
        else
            text = string_gsub(text, "<pet>", "")
        end
        if self:unitIsNPC(unit) then
            color = "ffff9a"
            text = string_gsub(text, "<npc>", "|cff"..color.."NPC".."|r")            
        else
            text = string_gsub(text, "<npc>", "")
        end
        font:SetText(text)
        font:SetWidth(2000)
        local w = font:GetStringWidth() + 10
        font:SetWidth(w)
        _G["DHUD2_"..unit.."_Text"]:SetWidth(w)
        _G["DHUD2_"..unit.."_Text"]:Show()
    else
        _G["DHUD2_"..unit.."_Text"]:Hide()
    end
end

function DHUD2:updateCastText(cast)
    local font = _G["DHUD2_castTime_Text_Text"]
    local castText = string_format("%.1f", cast)
    font:SetText(castText)
    font:SetWidth(2000)
    local w = font:GetStringWidth() + 10
    font:SetWidth(w)
    DHUD2_castTime_Text:SetWidth(w)
    if delayTime > 0 then
        font = _G["DHUD2_castDelay_Text_Text"]
        self:Debug("updateCastText: delay ", delayTime)
        local delayText = string_format("%.1f", delayTime)
        font:SetText(delayText)
        font:SetWidth(2000)
        w = font:GetStringWidth() + 10
        font:SetWidth(w)
        DHUD2_castDelay_Text:SetWidth(w)
        DHUD2_castDelay_Text:Show()
    else        
        DHUD2_castDelay_Text:Hide()
    end
end

function DHUD2:setFonts()
        local p = self.db.profile
        for i = 1, 16 do            
            local text = _G["DHUD2_playerBuff"..i.."_Text"]
            text:SetFont( defaultfont_num, p.fontsize.playerBuff / p.gral.scale, p.outlines.playerBuff)
        end
        DHUD2_target_Text_Text:SetFont(defaultfont, p.fontsize.targetName / p.gral.scale, p.outlines.target)
        DHUD2_targetTarget_Text_Text:SetFont(defaultfont, p.fontsize.targetTargetName / p.gral.scale, p.outlines.targetTarget)
        DHUD2_rct_Text_Text:SetFont(defaultfont, p.fontsize.castTime / p.gral.scale, p.outlines.castTime)
        DHUD2_lct_Text_Text:SetFont(defaultfont, p.fontsize.castTime / p.gral.scale, p.outlines.castTime)
        DHUD2_rcd_Text_Text:SetFont(defaultfont, p.fontsize.castDelay / p.gral.scale, p.outlines.castDelay)
        DHUD2_lcd_Text_Text:SetFont(defaultfont, p.fontsize.castDelay / p.gral.scale, p.outlines.castDelay)
        local bar = self.currLayout.playerHealth
        _G["DHUD2_"..bar.."_Text_Text"]:SetFont(defaultfont_num, p.fontsize.player / p.gral.scale, p.outlines.player)
        bar = self.currLayout.playerPower
        _G["DHUD2_"..bar.."_Text_Text"]:SetFont(defaultfont_num, p.fontsize.player / p.gral.scale, p.outlines.player)
        bar = self.currLayout.targetHealth
        _G["DHUD2_"..bar.."_Text_Text"]:SetFont(defaultfont_num, p.fontsize.target / p.gral.scale, p.outlines.target)
        bar = self.currLayout.targetPower
        _G["DHUD2_"..bar.."_Text_Text"]:SetFont(defaultfont_num, p.fontsize.target / p.gral.scale, p.outlines.target)
        bar = self.currLayout.petHealth
        _G["DHUD2_"..bar.."_Text_Text"]:SetFont(defaultfont_num, p.fontsize.pet / p.gral.scale, p.outlines.pet)
        bar = self.currLayout.petPower
        _G["DHUD2_"..bar.."_Text_Text"]:SetFont(defaultfont_num, p.fontsize.pet / p.gral.scale, p.outlines.pet)        bar = self.currLayout.targetTargetHealth
        if bar ~= "e" then
            _G["DHUD2_"..bar.."_Text_Text"]:SetFont(defaultfont_num, p.fontsize.targetTarget / p.gral.scale, p.outlines.targetTarget)
        end
        bar = self.currLayout.targetTargetPower
        if bar ~= "e" then
            _G["DHUD2_"..bar.."_Text_Text"]:SetFont(defaultfont_num, p.fontsize.targetTarget / p.gral.scale, p.outlines.targetTarget)
        end
        bar = self.currLayout.focusHealth
        if bar ~= "e" then
            _G["DHUD2_"..bar.."_Text_Text"]:SetFont(defaultfont_num, p.fontsize.focus / p.gral.scale, p.outlines.focus)
        end
        bar = self.currLayout.focusPower
        if bar ~= "e" then
            _G["DHUD2_"..bar.."_Text_Text"]:SetFont(defaultfont_num, p.fontsize.focus / p.gral.scale, p.outlines.focus)
        end
end
